@@ -13,6 +13,7 @@ __all__ = [
     'GreenSocket', '_GLOBAL_DEFAULT_TIMEOUT', 'set_nonblocking',
     'SOCKET_BLOCKING', 'SOCKET_CLOSED', 'CONNECT_ERR', 'CONNECT_SUCCESS',
     'shutdown_safe', 'SSL',
+    'socket_timeout',
 ]
 
 BUFFER_SIZE = 4096
@@ -27,6 +28,9 @@ if six.PY2:
 _original_socket = eventlet.patcher.original('socket').socket
 
 
+socket_timeout = eventlet.timeout.wrap_is_timeout(socket.timeout)
+
+
 def socket_connect(descriptor, address):
     """
     Attempts to connect to the address, returns the descriptor if it succeeds,
@@ -125,14 +129,14 @@ class GreenSocket(object):
     # This placeholder is to prevent __getattr__ from creating an infinite call loop
     fd = None
 
-    def __init__(self, family_or_realsock=socket.AF_INET, *args, **kwargs):
+    def __init__(self, family=socket.AF_INET, *args, **kwargs):
         should_set_nonblocking = kwargs.pop('set_nonblocking', True)
-        if isinstance(family_or_realsock, six.integer_types):
-            fd = _original_socket(family_or_realsock, *args, **kwargs)
+        if isinstance(family, six.integer_types):
+            fd = _original_socket(family, *args, **kwargs)
             # Notify the hub that this is a newly-opened socket.
             notify_opened(fd.fileno())
         else:
-            fd = family_or_realsock
+            fd = family
 
         # import timeout from other socket, if it was there
         try:
@@ -210,14 +214,14 @@ class GreenSocket(object):
         if self.act_non_blocking:
             return self.fd.accept()
         fd = self.fd
+        _timeout_exc = socket_timeout('timed out')
         while True:
             res = socket_accept(fd)
             if res is not None:
                 client, addr = res
                 set_nonblocking(client)
                 return type(self)(client), addr
-            self._trampoline(fd, read=True, timeout=self.gettimeout(),
-                             timeout_exc=socket.timeout("timed out"))
+            self._trampoline(fd, read=True, timeout=self.gettimeout(), timeout_exc=_timeout_exc)
 
     def _mark_as_closed(self):
         """ Mark this socket as being closed """
@@ -233,6 +237,7 @@ class GreenSocket(object):
         if self.act_non_blocking:
             return self.fd.connect(address)
         fd = self.fd
+        _timeout_exc = socket_timeout('timed out')
         if self.gettimeout() is None:
             while not socket_connect(fd, address):
                 try:
@@ -246,10 +251,10 @@ class GreenSocket(object):
                 if socket_connect(fd, address):
                     return
                 if time.time() >= end:
-                    raise socket.timeout("timed out")
+                    raise _timeout_exc
+                timeout = end - time.time()
                 try:
-                    self._trampoline(fd, write=True, timeout=end - time.time(),
-                                     timeout_exc=socket.timeout("timed out"))
+                    self._trampoline(fd, write=True, timeout=timeout, timeout_exc=_timeout_exc)
                 except IOClosed:
                     # ... we need some workable errno here.
                     raise socket.error(errno.EBADFD)
@@ -270,14 +275,15 @@ class GreenSocket(object):
                     return errno.EBADFD
         else:
             end = time.time() + self.gettimeout()
+            timeout_exc = socket.timeout(errno.EAGAIN)
             while True:
                 try:
                     if socket_connect(fd, address):
                         return 0
                     if time.time() >= end:
-                        raise socket.timeout(errno.EAGAIN)
+                        raise timeout_exc
                     self._trampoline(fd, write=True, timeout=end - time.time(),
-                                     timeout_exc=socket.timeout(errno.EAGAIN))
+                                     timeout_exc=timeout_exc)
                     socket_checkerr(fd)
                 except socket.error as ex:
                     return get_errno(ex)
@@ -299,6 +305,11 @@ class GreenSocket(object):
             res = _python2_fileobject(dupped, *args, **kwargs)
             if hasattr(dupped, "_drop"):
                 dupped._drop()
+                # Making the close function of dupped None so that when garbage collector
+                # kicks in and tries to call del, which will ultimately call close, _drop
+                # doesn't get called on dupped twice as it has been already explicitly called in
+                # previous line
+                dupped.close = None
             return res
 
     def makeGreenFile(self, *args, **kw):
@@ -311,9 +322,9 @@ class GreenSocket(object):
             self.fd,
             read=True,
             timeout=self.gettimeout(),
-            timeout_exc=socket.timeout("timed out"))
+            timeout_exc=socket_timeout('timed out'))
 
-    def _recv_loop(self, recv_meth, *args):
+    def _recv_loop(self, recv_meth, empty_val, *args):
         fd = self.fd
         if self.act_non_blocking:
             return recv_meth(*args)
@@ -335,7 +346,7 @@ class GreenSocket(object):
                 if get_errno(e) in SOCKET_BLOCKING:
                     pass
                 elif get_errno(e) in SOCKET_CLOSED:
-                    return b''
+                    return empty_val
                 else:
                     raise
 
@@ -346,22 +357,23 @@ class GreenSocket(object):
                 raise EOFError()
 
     def recv(self, bufsize, flags=0):
-        return self._recv_loop(self.fd.recv, bufsize, flags)
+        return self._recv_loop(self.fd.recv, b'', bufsize, flags)
 
     def recvfrom(self, bufsize, flags=0):
-        return self._recv_loop(self.fd.recvfrom, bufsize, flags)
+        return self._recv_loop(self.fd.recvfrom, b'', bufsize, flags)
 
     def recv_into(self, buffer, nbytes=0, flags=0):
-        return self._recv_loop(self.fd.recv_into, buffer, nbytes, flags)
+        return self._recv_loop(self.fd.recv_into, 0, buffer, nbytes, flags)
 
     def recvfrom_into(self, buffer, nbytes=0, flags=0):
-        return self._recv_loop(self.fd.recvfrom_into, buffer, nbytes, flags)
+        return self._recv_loop(self.fd.recvfrom_into, 0, buffer, nbytes, flags)
 
     def _send_loop(self, send_method, data, *args):
         if self.act_non_blocking:
             return send_method(data, *args)
 
-        while 1:
+        _timeout_exc = socket_timeout('timed out')
+        while True:
             try:
                 return send_method(data, *args)
             except socket.error as e:
@@ -371,7 +383,7 @@ class GreenSocket(object):
 
             try:
                 self._trampoline(self.fd, write=True, timeout=self.gettimeout(),
-                                 timeout_exc=socket.timeout("timed out"))
+                                 timeout_exc=_timeout_exc)
             except IOClosed:
                 raise socket.error(errno.ECONNRESET, 'Connection closed by another thread')
 
@@ -478,5 +490,5 @@ def shutdown_safe(sock):
     except socket.error as e:
         # we don't care if the socket is already closed;
         # this will often be the case in an http server context
-        if get_errno(e) not in (errno.ENOTCONN, errno.EBADF):
+        if get_errno(e) not in (errno.ENOTCONN, errno.EBADF, errno.ENOTSOCK):
             raise
